#!/usr/bin/perl -w

######################################################
#
#	3D wrapper exploration
#
#	The 3D wrapper has the ability to produce a 3D exploration
#	by doing a sweep on the x-axis and y-axis for 2 input parameters
#	and show the normalized value of an output parameter on the z-axis
#
#	The results are printed in a file <output_file_name>.txt
#	and a graph can be plot in a file <output_file_name>.eps
#
#	How to use it:
#	./3Dexploration.pl	<dc config file>								datacenter configuration file
#						<resource config file>							resource configuration file
#						<output_file_name>								filename of the output file without extension (the extensions are .txt and .eps)
#						<output_result[:reference value]>				output value name to use
#																		optional: the reference value is optional if you want provide a value comming from another configuration
#																		in that case you can put 1 to obtain absolute values
#						<x_parameter_name>								parameter x name 
#						<x_parameter_value_ref>							reference value of x 
#						<x_parameter_range_min>							minimale value of the x range
#						<x_parameter_range_max>							maximale value of the x range
#						<x_parameter_range_step>						number of steps in the x range
#						<x_parameter_range_type: absolute/relative>		the range can be absolute or relative
#						<y_parameter_name>								parameter y name 
#						<y_parameter_value_ref>							reference value of y 
#						<y_parameter_range_min>							minimale value of the y range
#						<y_parameter_range_max>							maximale value of the y range
#						<y_parameter_range_step>						number of steps in the y range
#						<y_parameter_range_type: absolute/relative>		the range can be absolute or relative
#						[xaxis:name]									optional to put a name 'name' for the x-axis of the graph (by default <x_parameter_name> is used)
#						[yaxis:name]									optional to put a name 'name' for the y-axis of the graph (by default <y_parameter_name> is used)
#						[zaxis:name]									optional to put a name 'name' for the z-axis of the graph (by default <output_result> is used)
#						[nognuplot]										optional: put "nognuplot" if you do not want to get the graph
#
#																
#
#  Example of command:
# ./3Dexploration.pl template/dc_template.params template/r_template.params tco tco c_chip 100 50 200 10 absolute n_srvmodulesreq 50000 25000 100000 25000 absolute xaxis:"Processor cost" yaxis:"#servers" zaxis:"Normalized TCO"
#
######################################################


use strict;

use POSIX qw/ceil/;

use File::Copy;

use lib "../lib/"; 
use helper;


######################################################
#
# Get input parameters
#
######################################################

my $argv_size = scalar @ARGV;
if ($argv_size < 16)
{
	print "usage: ./3Dexploration.pl <dc config file> <resource config file> <output_file_name> <output_result[:reference value]> <x_parameter_name> <x_parameter_value_ref> <x_parameter_range_min> <x_parameter_range_max> <x_parameter_range_step> <x_parameter_range_type: absolute/relative> <y_parameter_name> <y_parameter_value_ref> <y_parameter_range_min> <y_parameter_range_max> <y_parameter_range_step> <y_parameter_range_type: absolute/relative> [xaxis:name] [yaxis:name] [zaxis:name] [nognuplot]\n";
	exit(0);
}

my $dc_filename=$ARGV[0];
my $resource_filename=$ARGV[1];

my $filename= $ARGV[2];

my $z_name=$ARGV[3];

my $cst_defined=0;
my $z_ref=0;
my @tmp=split(':' ,$z_name);
my $tmp_size=scalar @tmp;
if($tmp_size==2)
{
	$z_name=$tmp[0];
	$z_ref=$tmp[1];
	$cst_defined=1;
}
elsif($tmp_size==1){;}
else{print("error: $z_name not correctly defined\n");exit(0);}

my $x_name=$ARGV[4];
my $x_ref=$ARGV[5];
my $x_min=$ARGV[6];
my $x_max=$ARGV[7];
my $x_step=$ARGV[8];
my $x_relative;
if($ARGV[9] eq "relative")
{
	$x_relative=1;
}
elsif($ARGV[9] eq "absolute")
{
	$x_relative=0;
}
else{print("error: x_parameter_range_type has to be relative or absolute\n");exit(0);}

my $y_name=$ARGV[10];
my $y_ref=$ARGV[11];
my $y_min=$ARGV[12];
my $y_max=$ARGV[13];
my $y_step=$ARGV[14];
my $y_relative;
if($ARGV[15] eq "relative")
{
	$y_relative=1;
}
elsif($ARGV[15] eq "absolute")
{
	$y_relative=0;
}
else{print("error: y_parameter_range_type has to be relative or absolute\n");exit(0);}

my $x_axis_name=$x_name;
my $y_axis_name=$y_name;
my $z_axis_name=$z_name;
my $gnuplot=1;
for(my $i=16;$i<20;$i++)
{
	if(defined($ARGV[$i]))
	{
		my $arg_tmp=$ARGV[$i];
		
		my @list=split(':',$arg_tmp);
		
		if( $list[0] eq "nognuplot")
		{
			$gnuplot=0;
		}
		elsif($list[0] eq "xaxis")
		{
			$x_axis_name=$list[1];
		}
		elsif($list[0] eq "yaxis")
		{
			if(scalar(@list)!=2){print ("Error: parameter $arg_tmp not correctly defined\n");exit(0);}
			$y_axis_name=$list[1];
		}
		elsif($list[0] eq "zaxis")
		{
			if(scalar(@list)!=2){print ("Error: parameter $arg_tmp not correctly defined\n");exit(0);}
			$z_axis_name=$list[1];
		}
		else{print ("Error: parameter $arg_tmp not correctly defined\n");exit(0);}
	}
	else{last;}
}


######################################################
#
# General Initialization
#
######################################################

#create the helper
my $helper = helper->new("");

#check that the inputs are correct
if($helper->checkInputParameterName($x_name)==-1){print("Execution stopped\n");exit(0); }
if($helper->checkInputParameterName($y_name)==-1){print("Execution stopped\n");exit(0); }
#check that the output name is correct
if($helper->checkOutputValueName($z_name)==-1){print("Execution stopped\n");exit(0); }
if($helper->checkOutputValueName($z_name)==1){print("Warning: The parameter 'report_resources' in the DC file must be set to 1 to get correct results\n"); }




#to get the current dir
my $currentdir=`pwd`;
chomp $currentdir;

#open outputfile
open(OUTPUTFILE,">$currentdir/output/$filename.txt") || die ("Error when opening $currentdir/output/$filename.txt") ;


######################################################
#
# Specific Initialization
#
######################################################

#get the parameters from files
my $ptr_dc=$helper->parserDCFile("$dc_filename");
my %dc=%{$ptr_dc};

my $ptr_resource=$helper->parserResourceFile("$resource_filename");
my %resource=%{$ptr_resource};

	####################################################
	#get the reference value 
	#(if needed i.e. $cst_defined==0)
	####################################################

	if($cst_defined==0)
	{
		#generate the current configuration file
		$dc{"$x_name"}=$x_ref;
		$dc{"$y_name"}=$y_ref;
		$helper->createParameterFile("$currentdir/dc.params",\%dc);

		$resource{"$x_name"}=$x_ref;
		$resource{"$y_name"}=$y_ref;
		$helper->createParameterFile("$currentdir/s.params",\%resource);

		#call the eetco kernel and keep the output
		$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/ref");

		#get the result value
		my $ptr_res=$helper->parserResultKernel("$currentdir/output/ref");
		my %res=%{$ptr_res};
		$z_ref=$res{"$z_name"};
	}

####################################################
# main exploration loop
####################################################

#exploration loop of x
for(my $x=$x_min;$x<=$x_max;$x+=$x_step)
{
	my $x_value=$x;	
	if($x_relative==1)
	{
		$x_value=$x * $x_ref;
	}
	
	#exploration loop of y
	for(my $y=$y_min;$y<=$y_max;$y+=$y_step)
	{
		my $y_value=$y;		
		if($y_relative==1)
		{
			$y_value=$y * $y_ref;
		}
		
		#generate the current configuration file
		$dc{"$x_name"}=$x_value;
		$dc{"$y_name"}=$y_value;
		$helper->createParameterFile("$currentdir/dc.params",\%dc);
		
		$resource{"$x_name"}=$x_value;
		$resource{"$y_name"}=$y_value;
		$helper->createParameterFile("$currentdir/s.params",\%resource);

		#call the eetco kernel and keep the output
		$helper->callKernel("$currentdir/dc.params","$currentdir/s.params","$currentdir/output/x_$x_value._y_$y_value");
	
		#get the result value
		my $ptr_res=$helper->parserResultKernel("$currentdir/output/x_$x_value._y_$y_value");
		my %res=%{$ptr_res};
		my $z=$res{"$z_name"};
		
		my $normalized_z=$z/$z_ref;
		
		print(OUTPUTFILE "$x \t $y \t $normalized_z \n");
	}
	print(OUTPUTFILE "\n");#for gnuplot 3D plot
}

close(OUTPUTFILE);

#keep the output file in the current directory
copy("$currentdir/output/$filename.txt","$currentdir/$filename.txt") or die "Copy failed: $!";

####################################################
# gnulpot call
####################################################
if($gnuplot==1)
{
	$helper->makeGnuplotCurves3D("$currentdir/$filename.txt","$currentdir","$filename","$x_axis_name","$y_axis_name","$z_axis_name");
}

####################################################
# clean
####################################################
$helper->deleteFile("$currentdir/dc.params");
$helper->deleteFile("$currentdir/s.params");
$helper->deleteFileInDir("$currentdir/output/");

